@@ -357,33 +357,40 @@ class OAuthRemoteApp(object):
             return resp, content
 
     def get(self, *args, **kwargs):
-        """Sends a ``GET`` request. Accepts the same paramters as
+        """Sends a ``GET`` request. Accepts the same parameters as
         :meth:`request`.
         """
         kwargs['method'] = 'GET'
         return self.request(*args, **kwargs)
 
     def post(self, *args, **kwargs):
-        """Sends a ``POST`` request. Accepts the same paramters as
+        """Sends a ``POST`` request. Accepts the same parameters as
         :meth:`request`.
         """
         kwargs['method'] = 'POST'
         return self.request(*args, **kwargs)
 
     def put(self, *args, **kwargs):
-        """Sends a ``PUT`` request. Accepts the same paramters as
+        """Sends a ``PUT`` request. Accepts the same parameters as
         :meth:`request`.
         """
         kwargs['method'] = 'PUT'
         return self.request(*args, **kwargs)
 
     def delete(self, *args, **kwargs):
-        """Sends a ``DELETE`` request. Accepts the same paramters as
+        """Sends a ``DELETE`` request. Accepts the same parameters as
         :meth:`request`.
         """
         kwargs['method'] = 'DELETE'
         return self.request(*args, **kwargs)
 
+    def patch(self, *args, **kwargs):
+        """Sends a ``PATCH`` request. Accepts the same parameters as
+        :meth:`post`.
+        """
+        kwargs['method'] = 'PATCH'
+        return self.request(*args, **kwargs)
+
     def request(self, url, data=None, headers=None, format='urlencoded',
                 method='GET', content_type=None, token=None):
         """
@@ -619,7 +626,7 @@ class OAuthRemoteApp(object):
         """Handles a unknown authorization response."""
         return None
 
-    def handle_response(self):
+    def authorized_response(self):
         """Handles authorization response smartly."""
         if 'oauth_verifier' in request.args:
             try:
@@ -640,10 +647,18 @@ class OAuthRemoteApp(object):
         return data
 
     def authorized_handler(self, f):
-        """Handles an OAuth callback."""
+        """Handles an OAuth callback.
+
+        .. versionchanged:: 0.7
+           @authorized_handler is deprecated in favor for authorized_response.
+        """
         @wraps(f)
         def decorated(*args, **kwargs):
-            data = self.handle_response()
+            log.warn(
+                '@authorized_handler is deprecated in favor for '
+                'authorized_response'
+            )
+            data = self.authorized_response()
             return f(*((data,) + args), **kwargs)
         return decorated
 
