@@ -16,7 +16,7 @@ from flask import make_response, abort
 from oauthlib.oauth1 import RequestValidator
 from oauthlib.oauth1 import WebApplicationServer as Server
 from oauthlib.oauth1 import SIGNATURE_HMAC, SIGNATURE_RSA
-from oauthlib.common import to_unicode, add_params_to_uri
+from oauthlib.common import to_unicode, add_params_to_uri, urlencode
 from oauthlib.oauth1.rfc5849 import errors
 from ..utils import extract_params, create_response
 
@@ -408,6 +408,10 @@ class OAuth1Provider(object):
                 return redirect(e.in_uri(self.error_uri))
             except errors.InvalidClientError as e:
                 return redirect(e.in_uri(self.error_uri))
+            except Exception as e:
+                return redirect(add_params_to_uri(
+                    self.error_uri, {'error': e.message}
+                ))
         return decorated
 
     def confirm_authorization_request(self):
@@ -428,6 +432,10 @@ class OAuth1Provider(object):
             return redirect(e.in_uri(self.error_uri))
         except errors.InvalidClientError as e:
             return redirect(e.in_uri(self.error_uri))
+        except Exception as e:
+            return redirect(add_params_to_uri(
+                self.error_uri, {'error': e.message}
+            ))
 
     def request_token_handler(self, f):
         """Request token handler decorator.
@@ -454,6 +462,10 @@ class OAuth1Provider(object):
                 return create_response(*ret)
             except errors.OAuth1Error as e:
                 return _error_response(e)
+            except Exception as e:
+                e.urlencoded = urlencode([('error', str(e))])
+                e.status_code = 400
+                return _error_response(e)
         return decorated
 
     def access_token_handler(self, f):
@@ -481,6 +493,10 @@ class OAuth1Provider(object):
                 return create_response(*ret)
             except errors.OAuth1Error as e:
                 return _error_response(e)
+            except Exception as e:
+                e.urlencoded = urlencode([('error', str(e))])
+                e.status_code = 400
+                return _error_response(e)
         return decorated
 
     def require_oauth(self, *realms, **kwargs):
@@ -496,10 +512,14 @@ class OAuth1Provider(object):
 
                 server = self.server
                 uri, http_method, body, headers = extract_params()
-                valid, req = server.validate_protected_resource_request(
-                    uri, http_method, body, headers, realms
-                )
-
+                try:
+                    valid, req = server.validate_protected_resource_request(
+                        uri, http_method, body, headers, realms
+                    )
+                except Exception as e:
+                    e.urlencoded = urlencode([('error', str(e))])
+                    e.status_code = 400
+                    return _error_response(e)
                 for func in self._after_request_funcs:
                     valid, req = func(valid, req)
 
