@@ -18,7 +18,7 @@ from werkzeug import cached_property
 from werkzeug.utils import import_string
 from oauthlib import oauth2
 from oauthlib.oauth2 import RequestValidator, Server
-from oauthlib.common import to_unicode
+from oauthlib.common import to_unicode, add_params_to_uri
 from ..utils import extract_params, decode_base64, create_response
 
 __all__ = ('OAuth2Provider', 'OAuth2RequestValidator')
@@ -249,13 +249,22 @@ class OAuth2Provider(object):
     def usergetter(self, f):
         """Register a function as the user getter.
 
-        This decorator is only required for password credential
+        This decorator is only required for **password credential**
         authorization::
 
             @oauth.usergetter
-            def get_user(username=username, password=password,
+            def get_user(username, password, client, request,
                          *args, **kwargs):
-                return get_user_by_username(username, password)
+                # client: current request client
+                if not client.has_password_credential_permission:
+                    return None
+                user = User.get_user_by_username(username)
+                if not user.validate_password(password):
+                    return None
+
+                # parameter `request` is an OAuthlib Request object.
+                # maybe you will need it somewhere
+                return user
         """
         self._usergetter = f
         return f
@@ -375,6 +384,10 @@ class OAuth2Provider(object):
                 except oauth2.FatalClientError as e:
                     log.debug('Fatal client error %r', e)
                     return redirect(e.in_uri(self.error_uri))
+                except Exception as e:
+                    return redirect(add_params_to_uri(
+                        self.error_uri, {'error': e.message}
+                    ))
 
             else:
                 redirect_uri = request.values.get('redirect_uri', None)
@@ -384,6 +397,10 @@ class OAuth2Provider(object):
             except oauth2.FatalClientError as e:
                 log.debug('Fatal client error %r', e)
                 return redirect(e.in_uri(self.error_uri))
+            except Exception as e:
+                return redirect(add_params_to_uri(
+                    self.error_uri, {'error': e.message}
+                ))
 
             if not isinstance(rv, bool):
                 # if is a response or redirect
@@ -421,6 +438,10 @@ class OAuth2Provider(object):
             return redirect(e.in_uri(self.error_uri))
         except oauth2.OAuth2Error as e:
             return redirect(e.in_uri(redirect_uri))
+        except Exception as e:
+            return redirect(add_params_to_uri(
+                self.error_uri, {'error': e.message}
+            ))
 
     def token_handler(self, f):
         """Access/refresh token handler decorator.
@@ -448,6 +469,38 @@ class OAuth2Provider(object):
             return create_response(*ret)
         return decorated
 
+    def revoke_handler(self, f):
+        """Access/refresh token revoke decorator.
+
+        Any return value by the decorated function will get discarded as
+        defined in [`RFC7009`_].
+
+        You can control the access method with the standard flask routing
+        mechanism, as per [`RFC7009`_] it is recommended to only allow
+        the `POST` method::
+
+            @app.route('/oauth/revoke', methods=['POST'])
+            @oauth.revoke_handler
+            def revoke_token():
+                pass
+
+        .. _`RFC7009`: http://tools.ietf.org/html/rfc7009
+        """
+        @wraps(f)
+        def decorated(*args, **kwargs):
+            server = self.server
+
+            token = request.values.get('token')
+            request.token_type_hint = request.values.get('token_type_hint')
+            if token:
+                request.token = token
+
+            uri, http_method, body, headers = extract_params()
+            ret = server.create_revocation_response(
+                uri, headers=headers, body=body, http_method=http_method)
+            return create_response(*ret)
+        return decorated
+
     def require_oauth(self, *scopes):
         """Protect resource with specified scopes."""
         def wrapper(f):
@@ -514,7 +567,7 @@ class OAuth2RequestValidator(RequestValidator):
             return True
         auth_required = ('authorization_code', 'refresh_token')
         return 'Authorization' in request.headers and\
-                request.grant_type in auth_required
+            request.grant_type in auth_required
 
     def authenticate_client(self, request, *args, **kwargs):
         """Authenticate itself in other means.
@@ -794,7 +847,7 @@ class OAuth2RequestValidator(RequestValidator):
         redirect_uris strictly, you can add a `validate_redirect_uri`
         function on grant for a customized validation.
         """
-        request.client = request.client = self._clientgetter(client_id)
+        request.client = request.client or self._clientgetter(client_id)
         client = request.client
         if hasattr(client, 'validate_redirect_uri'):
             return client.validate_redirect_uri(redirect_uri)
@@ -858,3 +911,24 @@ class OAuth2RequestValidator(RequestValidator):
             return False
         log.debug('Password credential authorization is disabled.')
         return False
+
+    def revoke_token(self, token, token_type_hint, request, *args, **kwargs):
+        """Revoke an access or refresh token.
+        """
+        if token_type_hint:
+            tok = self._tokengetter(**{token_type_hint: token})
+        else:
+            tok = self._tokengetter(access_token=token)
+            if not tok:
+                tok = self._tokengetter(refresh_token=token)
+
+        if tok and tok.client_id == request.client.client_id:
+            request.client_id = tok.client_id
+            request.user = tok.user
+            tok.delete()
+            return True
+
+        msg = 'Invalid token supplied.'
+        log.debug(msg)
+        request.error_message = msg
+        return False
